@@ -8,37 +8,37 @@
 
 fd <- function (coef=NULL, basisobj=NULL, fdnames=NULL)
 {
-  #  This function creates a functional data object.
-  #    A functional data object consists of a basis for expanding a functional
-  #    observation and a set of coefficients defining this expansion.
-  #    The basis is contained in a "basisfd" object that is, a realization
-  #    of the "basisfd" class.
+# This function creates a functional data object.
+#    A functional data object consists of a basis for expanding a functional
+#    observation and a set of coefficients defining this expansion.
+#    The basis is contained in a "basisfd" object that is, a realization
+#    of the "basisfd" class.
 
-  #  Arguments
-  #  COEF ... An array containing coefficient values for the expansion of each
-  #             set of function values in terms of a set of basis functions.
-  #           If COEF is a three-way array, then the first dimension
-  #             corresponds to basis functions, the second to replications,
-  #             and the third to variables.
-  #           If COEF is a matrix, it is assumed that there is only
-  #             one variable per replication, and then
-  #                 rows    correspond to basis functions
-  #                 columns correspond to replications
-  #           If COEF is a vector, it is assumed that there is only one
-  #             replication and one variable.
-  #  BASISOBJ ... a functional data basis object
-  #  FDNAMES  ... The analogue of the dimnames attribute of an array, this is
-  #               a list of length 3 with members containing:
-  #               1. a character vector of names for the argument values
-  #               2. a character vector of names for the replications or cases
-  #               3. a character vector of names for the functions
-  #               Each of these vectors can have a name referring to the modality
-  #                 of the data.  An example would be "time", "reps", "values"
+#  Arguments
+#  COEF ... An array containing coefficient values for the expansion of each
+#             set of function values in terms of a set of basis functions.
+#           If COEF is a three-way array, then the first dimension
+#             corresponds to basis functions, the second to replications,
+#             and the third to variables.
+#           If COEF is a matrix, it is assumed that there is only
+#             one variable per replication, and then
+#                 rows    correspond to basis functions
+#                 columns correspond to replications
+#           If COEF is a vector, it is assumed that there is only one
+#             replication and one variable.
+#  BASISOBJ ... a functional data basis object
+#  FDNAMES  ... The analogue of the dimnames attribute of an array, this is
+#               a list of length 3 with members containing:
+#               1. a character vector of names for the argument values
+#               2. a character vector of names for the replications or cases
+#               3. a character vector of names for the functions
+#               Each of these vectors can have a name referring to the modality
+#                 of the data.  An example would be "time", "reps", "values"
 
-  #  Returns:
-  #  FD ... a functional data object
+#  Returns:
+#  FD ... a functional data object
 
-  #  Last modified 3 February 2010 by Jim Ramsay
+#  Last modified 3 February 2010 by Jim Ramsay
 
 ##
 ## 1.  check coef and get its dimensions
@@ -427,10 +427,10 @@ minus.fd <- function(e1, e2, basisobj=NULL)
     return(e1)
   }
 #
-if (!(inherits(e1, "fd") || inherits(e2, "fd")))
+  if(!(inherits(e1, "fd") || inherits(e2, "fd")))
       stop("Neither argument for - is a functional data object.")
 
-if (inherits(e1, "fd") && inherits(e2, "fd")) {
+  if(inherits(e1, "fd") && inherits(e2, "fd")) {
     #  both arguments are functional data objects
     #  check to see of the two bases are identical
     #  and if (the coefficient matrices are conformable.
@@ -669,7 +669,7 @@ times.fd <- function(e1, e2, basisobj=NULL)
     #  e1 is single,  e2 has replications
 
     if (coefd1[2] == 1 && coefd2[2] > 1) {
-      if     (ndim1 == 2) {
+      if (ndim1 == 2) {
         coef1 <- matrix(coef1,coefd1[1],coefd2[2])
       } else if (ndim1 == 3) {
         temp <- array(0,coefd2)
@@ -720,10 +720,6 @@ times.fd <- function(e1, e2, basisobj=NULL)
     nbasis1   <- basisobj1$nbasis
     nbasis2   <- basisobj2$nbasis
 
-    #  set default basis object
-
-    if(is.null(basisobj)) basisobj <- basisobj1*basisobj2
-
     #  check that the ranges match if a range not supplied
 
     rangeval1 <- basisobj1$rangeval
@@ -731,6 +727,10 @@ times.fd <- function(e1, e2, basisobj=NULL)
     if (any(rangeval1 != rangeval2))
       stop("The ranges of the arguments are not equal.")
 
+    #  set default basis object
+
+    if(is.null(basisobj)) basisobj <- basisobj1*basisobj2
+
     #  set up a fine mesh for evaluating the product
 
     neval   <- max(10*max(nbasis1,nbasis2) + 1, 201)
@@ -781,7 +781,7 @@ times.fd <- function(e1, e2, basisobj=NULL)
     if ((!(is.numeric(e1) || is.numeric(e2))))
       stop("Neither argument for * is numeric.")
     #  make the numerical factor the first objec
-    if        (is.numeric(e1) && inherits(e2, "fd")) {
+    if (is.numeric(e1) && inherits(e2, "fd")) {
       fac   <- e1
       fdobj <- e2
     } else if (is.numeric(e2) && inherits(e1, "fd")) {
@@ -814,247 +814,10 @@ times.fd <- function(e1, e2, basisobj=NULL)
 }
 
 #  -------------------------------------------------------------------
-#  power method for "fd"
+#  power & sqrt methods for "fd"
 #  -------------------------------------------------------------------
 
-"^.fd" <- function(e1, e2)
-{
-#  A positive integer pointwise power of a functional data object with
-#  a B-splinebasis.  powerfd = fdobj^a.  
-#  Generic arguments e1 = fdobj and e2 = a.  
-#
-#  The basis is tested for being a B-spline basis.  The function then
-#  sets up a new spline basis with the same knots but with an order
-#  that is M-1 higher than the basis for FDOBJ, where M = ceiling(a),
-#  so that the order of differentiability for the new basis is
-#  appropriate in the event that a is a positive integer, and also
-#  to accommodate the additional curvature arising from taking a power.
-#  The power of the values of the function over a fine mesh are computed,
-#  and these are fit using the new basis.
-#
-#  Powers should be requested with caution, however, and especially if
-#  a < 1, because, if there is strong local curvature in FDOBJ,
-#  and if its basis is just barely adequate to capture this curvature,
-#  then the power of the function may have considerable error
-#  over this local area.  fdobj^a where a is close to zero is just
-#  such a situation.
-#
-#  If a power of a functional data object is required for which the
-#  basis is not a spline, it is better to either re-represent the
-#  function in a spline basis, or, perhaps even better, to do the
-#  math required to get the right basis and interpolate function
-#  values over a suitable mesh.  This is especially true for fourier
-#  bases.
-
-#  Last modified 3 Novem ber 2009
-
-#  check first two arguments
-
-fdobj = e1
-a     = e2
-tol   = 1e-4
-
-if ((!(inherits(fdobj, "fd"))))
-        stop("First argument for ^ is not a functional data object.")
-if ((!(is.numeric(a))))
-        stop("Second argument for ^ is not numeric.")
-
-#  extract basis
-
-basisobj = fdobj$basis
-
-#  test the basis for being of B-spline type
-
-if (basisobj$type != "bspline")
-    stop("FDOBJ does not have a spline basis.")
-
-
-nbasis        = basisobj$nbasis
-rangeval      = basisobj$rangeval
-interiorknots = basisobj$params
-norder        = nbasis - length(interiorknots)
-
-#  Number of points at which to evaluate the power.  Even low
-#  order bases can generate steep slopes and sharp curvatures,
-#  especially if powers less than 1 are involved.
-
-nmesh = max(10*nbasis+1,501)
-
-#  determine number curves and variables
-
-coefmat = fdobj$coef
-coefd   = dim(coefmat)
-ncurve  = coefd[2]
-if (length(coefd) == 2) {
-    nvar = 1
-} else {
-    nvar = coefd[3]
-}
-
-#  evaluate function over this mesh
-
-tval = seq(rangeval[1],rangeval[2],len=nmesh)
-fmat = eval.fd(tval, fdobj)
-
-#  find the minimum value over this mesh.  If the power is less than
-#  one, return an error message.
-
-fmin = min(c(fmat))
-
-#  a == 0:  set up a constant basis and return the unit function(s)
-
-if (a == 0) {
-    newbasis = create.constant.basis(rangeval)
-    if (nvar == 1) {
-      powerfd = fd(matrix(1,1,ncurve), newbasis)
-    } else {
-      powerfd = fd(array(1,c(1,ncurve,nvar)), newbasis)
-    }
-    return(powerfd)
-}
-
-#  a == 1:  return the function
-
-if (a == 1) {
-    powerfd = fdobj
-    return(powerfd)
-}
-
-#  Otherwise:
-
-m = ceiling(a)
-
-#  Check the size of the power.  If greater than one, estimating the
-#  functional data object is relatively safe since the curvatures
-#  involved are mild.  If not, then taking the power is a dangerous
-#  business.
-
-if (m == a && m > 1) {
-
-    #  a is an integer greater than one
-
-    newnorder = (norder-1)*m + 1
-    if (length(interiorknots) < 9) {
-        newbreaks = seq(rangeval[1], rangeval[2], len=11)
-    } else {
-        newbreaks = c(rangeval[1], interiorknots, rangeval[2])
-    }
-    nbreaks   = length(newbreaks)
-    newnbasis = newnorder + nbreaks - 2
-    newbasis  = create.bspline.basis(rangeval, newnbasis, newnorder, 
-                                     newbreaks)
-    ymat    = fmat^a
-    ytol    = max(abs(c(ymat)))*tol
-    powerfd = smooth.basis(tval, ymat, newbasis)$fd
-    ymathat = eval.fd(tval,powerfd)
-    ymatres = ymat - ymathat
-    maxerr  = max(abs(c(ymatres)))
-    while  (maxerr > ytol && nbreaks < nmesh) {
-        newnbasis = newnorder + nbreaks - 2
-        newbasis  = create.bspline.basis(rangeval, newnbasis, newnorder,
-                                         newbreaks)
-        newfdPar  = fdPar(newbasis, 2, 1e-20)
-        powerfd   = smooth.basis(tval, ymat, newfdPar)$fd
-        ymathat   = eval.fd(tval,powerfd)
-        ymatres   = ymat - ymathat
-        maxerr    = max(abs(c(ymatres)))
-        if (nbreaks*2 <= nmesh) {
-        newbreaks = sort(c(newbreaks,
-            (newbreaks[1:(nbreaks-1)]+newbreaks[2:nbreaks])/2))
-        } else {
-            newbreaks = tval
-        }
-        nbreaks = length(newbreaks)
-    }
-
-    if (maxerr > ytol)
-        warning("The maximum error exceeds the tolerance level.")
-
-    return(powerfd)
-
-} else {
-
-    #  a is fractional or negative
-
-    #  check for negative values and a fractional power
-
-    if (a > 0 && fmin < 0) stop(
-         paste("There are negative values",
-               "and the power is a positive fraction."))
-
-    #  check for zero or negative values and a negative power
-
-    if (a < 0 && fmin <= 0) stop(
-        paste("There are zero or negative values",
-               "and the power is negative."))
-
-    if (length(interiorknots) < 9) {
-        newbreaks = seq(rangeval[1], rangeval[2], n=11)
-    } else {
-        newbreaks = c(rangeval[1], interiorknots, rangeval[2])
-    }
-    nbreaks   = length(newbreaks)
-    newnorder = max(4, norder+m-1)
-    newnbasis = newnorder + nbreaks - 2
-    newbasis  = create.bspline.basis(rangeval, newnbasis, newnorder,
-                                     newbreaks)
-    nmesh     = max(10*nbasis+1,101)
-    tval      = seq(rangeval[1],rangeval[2],len=nmesh)
-    fmat      = eval.fd(tval, fdobj)
-    ymat      = fmat^a
-    ytol      = max(abs(c(ymat)))*tol
-    newfdPar  = fdPar(newbasis, 2, 1e-20)
-    powerfd   = smooth.basis(tval, ymat, newfdPar)$fd
-    ymathat   = eval.fd(tval,powerfd)
-    ymatres   = ymat - ymathat
-    maxerr    = max(abs(c(ymatres)))
-    while (maxerr > ytol && nbreaks < nmesh) {
-        newnbasis = newnorder + nbreaks - 2
-        newbasis  = create.bspline.basis(rangeval, newnbasis,
-                                         newnorder, newbreaks)
-        newfdPar  = fdPar(newbasis, 2, 1e-20)
-        powerfd   = smooth.basis(tval, ymat, newfdPar)$fd
-        ymathat   = eval.fd(tval,powerfd)
-        ymatres   = ymat - ymathat
-        maxerr    = max(abs(ymatres))*tol
-        if (nbreaks*2 <= nmesh) {
-            newbreaks = sort(c(newbreaks,
-            (newbreaks[1:(nbreaks-1)]+newbreaks[2:nbreaks])/2))
-        } else {
-            newbreaks = tval
-        }
-        nbreaks = length(newbreaks)
-    }
-
-    if (maxerr > ytol) {
-        warning("The maximum error exceeds the tolerance level.")
-    }
-
-    return(powerfd)
-
-}
-
-}
-
-#  -------------------------------------------------------------------
-#  sqrt method for "fd"
-#  -------------------------------------------------------------------
-
-sqrt.fd <- function(x)
-{
-#  Arguments:
-#  x ...  A functional data object
-#  Returns:
-#  FDAROOT  ...  A functional data object that is the square root of x
-#  Last modified:  27 october 2009
-
-    if ((!(inherits(x, "fd")))) stop(
-      "Argument for ^ is not a functional data object.")
-
-    fdaroot <- x^0.5
-
-    return(fdaroot)
-}
+# see exponentiate.fd
 
 #  ----------------------------------------------------------------
 #       mean for fd class
